import os
import json
import zlib
import base64

def load_fun_data(filename):
    file_path = os.path.join(os.path.dirname(__file__), filename + ".json.enc")

    if not os.path.exists(file_path):
        raise FileNotFoundError(f"⚠️ فایل {filename}.json موجود نیست!")

    with open(file_path, "r", encoding="utf-8") as f:
        encoded_data = f.read()

    # رمزگشایی از Base64
    compressed_data = base64.b64decode(encoded_data)

    # باز کردن فشرده‌سازی zlib
    json_data = zlib.decompress(compressed_data).decode("utf-8")

    return json.loads(json_data)



import pkgutil

def fun(filename: str) -> bytes:
    return pkgutil.get_data(__name__, f"fun/{filename}")

def list_fun() -> list[str]:
    return [
        "alaki.json.enc",
        "chandsaletbod.json.enc",
        "chanvaghte.json.enc",
        "dastan.json.enc",
        "deghatkardin.json.enc",
        "dialog.json.enc",
        "eteraf.json.enc",
        "fantezi.json.enc",
        "khaterat.json.enc",
        "mrghazi.json.enc",
        "panapa.json.enc",
        "ravanshenasi.json.enc",
    ]



import json
import os
from typing import Dict, List, Optional, Union, Any
from uuid import uuid4
import time
from copy import deepcopy

class Database:
    def __init__(self, name: str = "database", autocommit: bool = True):
        self.name = name
        self.file_path = f"{name}.json"
        self.backup_path = f"{name}_backup.json"
        self.autocommit = autocommit
        self.transaction_stack = []
        self.data = self._load_database()
        if "_metadata" not in self.data:
            self.data["_metadata"] = {
                "version": "1.0",
                "created_at": time.time(),
                "last_modified": time.time()
            }
            self._commit()

    def _load_database(self) -> Dict:
        if os.path.exists(self.file_path):
            with open(self.file_path, 'r', encoding='utf-8') as file:
                data = json.load(file)
                if "tables" not in data:
                    data["tables"] = {}
                if "indexes" not in data:
                    data["indexes"] = {}
                if "_metadata" not in data:
                    data["_metadata"] = {
                        "version": "1.0",
                        "created_at": time.time(),
                        "last_modified": time.time()
                    }
                return data
        return {
            "tables": {},
            "indexes": {},
            "_metadata": {
                "version": "1.0",
                "created_at": time.time(),
                "last_modified": time.time()
            }
        }

    def _commit(self):
        if self.autocommit and not self.transaction_stack:
            if "_metadata" not in self.data:
                self.data["_metadata"] = {
                    "version": "1.0",
                    "created_at": time.time(),
                    "last_modified": time.time()
                }
            else:
                self.data["_metadata"]["last_modified"] = time.time()
            
            with open(self.file_path, 'w', encoding='utf-8') as file:
                json.dump(self.data, file, indent=4, ensure_ascii=False)

    def _create_backup(self):
        with open(self.backup_path, 'w', encoding='utf-8') as file:
            json.dump(self.data, file, indent=4, ensure_ascii=False)

    def restore_backup(self):
        if os.path.exists(self.backup_path):
            with open(self.backup_path, 'r', encoding='utf-8') as file:
                self.data = json.load(file)
            self._commit()
            return True
        return False

    def begin_transaction(self):
        self.transaction_stack.append(deepcopy(self.data))

    def commit_transaction(self):
        if self.transaction_stack:
            self.transaction_stack.pop()
            self._commit()

    def rollback_transaction(self):
        if self.transaction_stack:
            self.data = self.transaction_stack.pop()

    def create_table(self, table_name: str, table_type: str = "list", initial_data: Any = None) -> bool:
        if table_name in self.data["tables"]:
            return False
            
        if table_type == "list":
            self.data["tables"][table_name] = initial_data if initial_data is not None else []
        elif table_type == "dict":
            self.data["tables"][table_name] = initial_data if initial_data is not None else {}
        else:
            raise ValueError("Table type must be either 'list' or 'dict'")
            
        self._commit()
        return True

    def delete_table(self, table_name: str) -> bool:
        if table_name not in self.data["tables"]:
            return False
        del self.data["tables"][table_name]
        for index_name in list(self.data["indexes"].keys()):
            if index_name.startswith(f"{table_name}."):
                del self.data["indexes"][index_name]
                
        self._commit()
        return True

    def rename_table(self, old_name: str, new_name: str) -> bool:
        if old_name not in self.data["tables"] or new_name in self.data["tables"]:
            return False
            
        self.data["tables"][new_name] = self.data["tables"][old_name]
        del self.data["tables"][old_name]
        for index_name in list(self.data["indexes"].keys()):
            if index_name.startswith(f"{old_name}."):
                new_index_name = index_name.replace(f"{old_name}.", f"{new_name}.")
                self.data["indexes"][new_index_name] = self.data["indexes"][index_name]
                del self.data["indexes"][index_name]
                
        self._commit()
        return True

    def table_type(self, table_name: str) -> Optional[str]:
        if table_name not in self.data["tables"]:
            return None
            
        table_data = self.data["tables"][table_name]
        return "dict" if isinstance(table_data, dict) else "list"

    def convert_table_type(self, table_name: str, new_type: str) -> bool:
        if table_name not in self.data["tables"]:
            return False
            
        current_type = self.table_type(table_name)
        if current_type == new_type:
            return True
            
        try:
            if new_type == "list":
                if isinstance(self.data["tables"][table_name], dict):
                    self.data["tables"][table_name] = list(self.data["tables"][table_name].values())
            elif new_type == "dict":
                if isinstance(self.data["tables"][table_name], list):
                    new_dict = {}
                    for i, item in enumerate(self.data["tables"][table_name]):
                        if isinstance(item, dict) and "id" in item:
                            new_dict[item["id"]] = item
                        else:
                            new_dict[str(i)] = item
                    self.data["tables"][table_name] = new_dict
            else:
                return False
                
            self._commit()
            return True
        except Exception:
            return False

    def create_nested_table(self, parent_table: str, path: str, table_type: str = "list") -> bool:
        if parent_table not in self.data["tables"]:
            return False
            
        path_parts = path.split('.')
        current = self.data["tables"][parent_table]
        
        for part in path_parts[:-1]:
            if isinstance(current, dict):
                if part not in current:
                    current[part] = {}
                current = current[part]
            elif isinstance(current, list):
                try:
                    index = int(part)
                    if index < len(current):
                        current = current[index]
                    else:
                        return False
                except ValueError:
                    return False
            else:
                return False
                
        final_part = path_parts[-1]
        if isinstance(current, dict):
            if table_type == "list":
                current[final_part] = []
            else:
                current[final_part] = {}
            self._commit()
            return True
            
        return False

    def insert(self, table_name: str, data: Any, path: Optional[str] = None) -> bool:
        if table_name not in self.data["tables"]:
            return False
            
        if not path:
            if isinstance(self.data["tables"][table_name], list):
                self.data["tables"][table_name].append(data)
                self._commit()
                return True
            elif isinstance(self.data["tables"][table_name], dict):
                if isinstance(data, dict) and "id" in data:
                    self.data["tables"][table_name][data["id"]] = data
                else:
                    self.data["tables"][table_name][str(uuid4())] = data
                self._commit()
                return True
            return False
            
        path_parts = path.split('.')
        current = self.data["tables"][table_name]
        
        for part in path_parts[:-1]:
            if isinstance(current, dict):
                if part not in current:
                    return False
                current = current[part]
            elif isinstance(current, list):
                try:
                    index = int(part)
                    if index < len(current):
                        current = current[index]
                    else:
                        return False
                except ValueError:
                    return False
            else:
                return False
                
        final_part = path_parts[-1]
        if isinstance(current, dict):
            current[final_part] = data
            self._commit()
            return True
        elif isinstance(current, list):
            try:
                index = int(final_part)
                if index < len(current):
                    current[index] = data
                    self._commit()
                    return True
            except ValueError:
                pass
                
        return False

    def get(self, table_name: str, path: Optional[str] = None, default: Any = None) -> Any:
        if table_name not in self.data["tables"]:
            return default

        if not path:
            return deepcopy(self.data["tables"][table_name])

        try:
            path_parts = path.split('.')
            current = self.data["tables"][table_name]

            for part in path_parts:
                if isinstance(current, dict):
                    current = current.get(part, default)
                    if current is default:
                        return default
                elif isinstance(current, list):
                    try:
                        index = int(part)
                        current = current[index] if index < len(current) else default
                    except (ValueError, IndexError):
                        return default
                else:
                    return default

            return deepcopy(current)
        except Exception:
            return default

    def update(self, table_name: str, new_data: Any, path: Optional[str] = None) -> bool:
        if table_name not in self.data["tables"]:
            return False
            
        if not path:
            self.data["tables"][table_name] = new_data
            self._commit()
            return True
            
        path_parts = path.split('.')
        current = self.data["tables"][table_name]
        
        for part in path_parts[:-1]:
            if isinstance(current, dict):
                if part not in current:
                    return False
                current = current[part]
            elif isinstance(current, list):
                try:
                    index = int(part)
                    if index < len(current):
                        current = current[index]
                    else:
                        return False
                except ValueError:
                    return False
            else:
                return False
                
        final_part = path_parts[-1]
        if isinstance(current, dict):
            current[final_part] = new_data
            self._commit()
            return True
        elif isinstance(current, list):
            try:
                index = int(final_part)
                if index < len(current):
                    current[index] = new_data
                    self._commit()
                    return True
            except ValueError:
                pass
                
        return False

    def delete(self, table_name: str, path: Optional[str] = None) -> bool:
        if table_name not in self.data["tables"]:
            return False
            
        if not path:
            self.data["tables"][table_name] = [] if isinstance(self.data["tables"][table_name], list) else {}
            self._commit()
            return True
            
        path_parts = path.split('.')
        current = self.data["tables"][table_name]
        
        for part in path_parts[:-1]:
            if isinstance(current, dict):
                if part not in current:
                    return False
                current = current[part]
            elif isinstance(current, list):
                try:
                    index = int(part)
                    if index < len(current):
                        current = current[index]
                    else:
                        return False
                except ValueError:
                    return False
            else:
                return False
                
        final_part = path_parts[-1]
        if isinstance(current, dict):
            if final_part in current:
                del current[final_part]
                self._commit()
                return True
        elif isinstance(current, list):
            try:
                index = int(final_part)
                if index < len(current):
                    del current[index]
                    self._commit()
                    return True
            except ValueError:
                pass
                
        return False

    def create_index(self, table_name: str, field: str) -> bool:
        if table_name not in self.data["tables"]:
            return False
            
        index_name = f"{table_name}.{field}"
        if index_name in self.data["indexes"]:
            return True
            
        table_data = self.data["tables"][table_name]
        index = {}
        
        if isinstance(table_data, list):
            for i, item in enumerate(table_data):
                if isinstance(item, dict) and field in item:
                    value = item[field]
                    if value not in index:
                        index[value] = []
                    index[value].append(i)
        elif isinstance(table_data, dict):
            for key, item in table_data.items():
                if isinstance(item, dict) and field in item:
                    value = item[field]
                    if value not in index:
                        index[value] = []
                    index[value].append(key)
                    
        self.data["indexes"][index_name] = index
        self._commit()
        return True

    def query(self, table_name: str, conditions: Dict, use_index: bool = True, limit: Optional[int] = None, sort: Optional[str] = None, reverse: bool = False) -> List[Any]:
        if table_name not in self.data["tables"]:
            return []

        table_data = self.data["tables"][table_name]
        results = []

        if use_index and conditions:
            first_field = next(iter(conditions))
            if isinstance(conditions[first_field], dict):
                operator, value = next(iter(conditions[first_field].items()))
                if operator == "$gte":
                    index_name = f"{table_name}.{first_field}"
                    if index_name in self.data["indexes"]:
                        index = self.data["indexes"][index_name]
                        filtered_keys = [k for k in index.keys() if int(k) >= value]
                        keys_or_indices = []
                        for k in filtered_keys:
                            keys_or_indices.extend(index[k])

                        if isinstance(table_data, list):
                            for i in keys_or_indices:
                                if i < len(table_data):
                                    results.append(deepcopy(table_data[i]))
                        elif isinstance(table_data, dict):
                            for key in keys_or_indices:
                                if key in table_data:
                                    results.append(deepcopy(table_data[key]))
                        return results[:limit] if limit is not None else results
            else:
                index_name = f"{table_name}.{first_field}"
                if index_name in self.data["indexes"]:
                    index = self.data["indexes"][index_name]
                    first_value = conditions[first_field]

                    if first_value in index:
                        keys_or_indices = index[first_value]
                        remaining_conditions = {k: v for k, v in conditions.items() if k != first_field}

                        if isinstance(table_data, list):
                            for i in keys_or_indices:
                                if i < len(table_data):
                                    item = table_data[i]
                                    if all(item.get(k) == v for k, v in remaining_conditions.items()):
                                        results.append(deepcopy(item))
                        elif isinstance(table_data, dict):
                            for key in keys_or_indices:
                                if key in table_data:
                                    item = table_data[key]
                                    if all(item.get(k) == v for k, v in remaining_conditions.items()):
                                        results.append(deepcopy(item))
                        return results[:limit] if limit is not None else results

        if isinstance(table_data, list):
            for item in table_data:
                if isinstance(item, dict):
                    match = True
                    for field, condition in conditions.items():
                        if isinstance(condition, dict):
                            for op, value in condition.items():
                                if op == "$gte":
                                    if not (item.get(field, 0) >= value):
                                        match = False
                                        break
                        else:
                            if item.get(field) != condition:
                                match = False
                                break
                    if match:
                        results.append(deepcopy(item))
        elif isinstance(table_data, dict):
            for item in table_data.values():
                if isinstance(item, dict):
                    match = True
                    for field, condition in conditions.items():
                        if isinstance(condition, dict):
                            for op, value in condition.items():
                                if op == "$gte":
                                    if not (item.get(field, 0) >= value):
                                        match = False
                                        break
                        else:
                            if item.get(field) != condition:
                                match = False
                                break
                    if match:
                        results.append(deepcopy(item))

        # اعمال مرتب‌سازی اگر مشخص شده باشد
        if sort:
            reverse_sort = reverse if isinstance(reverse, bool) else False
            results.sort(key=lambda x: x.get(sort, 0), reverse=reverse_sort)

        # اعمال محدودیت تعداد نتایج
        if limit is not None:
            results = results[:limit]

        return results

    def backup(self, backup_name: str) -> bool:
        try:
            backup_path = f"{self.name}_{backup_name}.json"
            with open(backup_path, 'w', encoding='utf-8') as file:
                json.dump(self.data, file, indent=4, ensure_ascii=False)
            return True
        except Exception:
            return False

    def vacuum(self) -> bool:
        try:
            for index_name in list(self.data["indexes"].keys()):
                table_name, field = index_name.split('.')
                if table_name in self.data["tables"]:
                    self.create_index(table_name, field)
            for table_name, table_data in self.data["tables"].items():
                if isinstance(table_data, list):
                    self.data["tables"][table_name] = [x for x in table_data if x is not None]
                    
            self._commit()
            return True
        except Exception:
            return False

    def list_tables(self) -> List[str]:
        return list(self.data["tables"].keys())

    def table_info(self, table_name: str) -> Optional[Dict]:
        if table_name not in self.data["tables"]:
            return None
            
        table_data = self.data["tables"][table_name]
        info = {
            "type": "dict" if isinstance(table_data, dict) else "list",
            "size": len(table_data),
            "indexes": [idx.split('.')[1] for idx in self.data["indexes"] if idx.startswith(f"{table_name}.")]
        }
        
        if isinstance(table_data, list) and table_data and isinstance(table_data[0], dict):
            info["fields"] = list(table_data[0].keys())
        elif isinstance(table_data, dict) and table_data and isinstance(next(iter(table_data.values())), dict):
            info["fields"] = list(next(iter(table_data.values())).keys())
            
        return info

    def export_table(self, table_name: str, format: str = "json") -> Optional[str]:
        if table_name not in self.data["tables"]:
            return None
            
        table_data = self.data["tables"][table_name]
        
        if format == "json":
            return json.dumps(table_data, indent=4, ensure_ascii=False)
        elif format == "csv":
            if isinstance(table_data, list) and table_data and isinstance(table_data[0], dict):
                import csv
                from io import StringIO
                
                output = StringIO()
                writer = csv.DictWriter(output, fieldnames=table_data[0].keys())
                writer.writeheader()
                writer.writerows(table_data)
                return output.getvalue()
            elif isinstance(table_data, dict) and table_data and isinstance(next(iter(table_data.values())), dict):
                import csv
                from io import StringIO
                
                first_item = next(iter(table_data.values()))
                output = StringIO()
                writer = csv.DictWriter(output, fieldnames=first_item.keys())
                writer.writeheader()
                writer.writerows(table_data.values())
                return output.getvalue()
            else:
                return None
        else:
            return None


import pkgutil

def fun(filename: str) -> bytes:
    return pkgutil.get_data(__name__, f"fun/{filename}")

def list_fun() -> list[str]:
    return [
        "alaki.json.enc",
        "chandsaletbod.json.enc",
        "chanvaghte.json.enc",
        "dastan.json.enc",
        "deghatkardin.json.enc",
        "dialog.json.enc",
        "eteraf.json.enc",
        "fantezi.json.enc",
        "khaterat.json.enc",
        "mrghazi.json.enc",
        "panapa.json.enc",
        "ravanshenasi.json.enc",
    ]


from .bot import (
    Bot, 
    CopyTextButton, 
    ChatPhoto, 
    InputMedia, 
    InputMediaPhoto,
    InputMediaVideo, 
    InputMediaAnimation, 
    InputMediaAudio, 
    InputMediaDocument, 
    InputFile,
    User,
    File
)
from .filter import Filters
from .ReplyKeyboard import ReplyKeyboardMarkup, KeyboardButton, WebAppInfo as ReplyWebAppInfo
from .InlineKeyboard import InlineKeyboardMarkup, InlineKeyboardButton, WebAppInfo as InlineWebAppInfo
from .update_wrapper import (
    UpdateWrapper, 
    CallbackQuery, 
    Message, 
    Chat,
    ChatMember,
    PhotoSize,
    Audio,
    Document,
    Voice,
    Location,
    Video,
    Invoice,
    Sticker
)
from .Database import Database
from .states import (
    State,
    TimedState,
    ChatState,
    StateMachine,
    AsyncState,
    AsyncChatState,
    StateManager
)

__all__ = [
    'Bot',
    'CopyTextButton',
    'ChatPhoto',
    'ReplyKeyboardMarkup', 
    'KeyboardButton',
    'InlineKeyboardMarkup', 
    'InlineKeyboardButton',
    'WebAppInfo',
    'InputMedia',
    'InputMediaPhoto',
    'InputMediaVideo',
    'InputMediaAnimation',
    'InputMediaAudio',
    'InputMediaDocument',
    'InputFile',
    'Filters',
    'Database',
    'State',
    'TimedState',
    'ChatState',
    'StateMachine',
    'AsyncState',
    'AsyncChatState',
    'StateManager',
    'UpdateWrapper',
    'CallbackQuery',
    'Message',
    'User',
    'Chat',
    'ChatParameter',
    'ChatMember',
    'PhotoSize',
    'Audio',
    'Document',
    'Voice',
    'Location',
    'Video',
    'Invoice',
    'Sticker',
    'SuccessfulPayment',
    'File'
]


import aiohttp
import asyncio
from typing import Callable, Optional, Dict, Any, List, Union

from .filter import Filters
from .update_wrapper import UpdateWrapper
from .InlineKeyboard import InlineKeyboardButton, InlineKeyboardMarkup
from .ReplyKeyboard import KeyboardButton, ReplyKeyboardMarkup
from .update_wrapper import CallbackQuery, Message
from .Database import Database

class User:
    def __init__(self, user_data: dict):
        self.id = user_data.get("id")
        self.is_bot = user_data.get("is_bot")
        self.first_name = user_data.get("first_name")
        self.last_name = user_data.get("last_name")
        self.username = user_data.get("username")
        self.language_code = user_data.get("language_code")

    def __str__(self):
        return (
            f"User(\n"
            f"    id={self.id},\n"
            f"    is_bot={self.is_bot},\n"
            f"    first_name={self.first_name},\n"
            f"    last_name={self.last_name},\n"
            f"    username={self.username},\n"
            f"    language_code={self.language_code}\n"
            f")"
        )

class File:
    def __init__(self, file_data: dict):
        self.file_id = file_data.get("file_id")
        self.file_unique_id = file_data.get("file_unique_id")
        self.file_size = file_data.get("file_size")
        self.file_path = file_data.get("file_path")

    def __str__(self):
        return (
            f"File(\n"
            f"    file_id={self.file_id},\n"
            f"    file_unique_id={self.file_unique_id},\n"
            f"    file_size={self.file_size},\n"
            f"    file_path={self.file_path}\n"
            f")"
        )

class SuccessfulPayment:
    def __init__(self, successful_payment_data: dict):
        self.currency = successful_payment_data.get("currency")
        self.total_amount = successful_payment_data.get("total_amount")
        self.invoice_payload = successful_payment_data.get("invoice_payload")
        self.telegram_payment_charge_id = successful_payment_data.get("telegram_payment_charge_id")

    def __str__(self):
        return (
            f"SuccessfulPayment(\n"
            f"    currency={self.currency},\n"
            f"    total_amount={self.total_amount},\n"
            f"    invoice_payload={self.invoice_payload},\n"
            f"    telegram_payment_charge_id={self.telegram_payment_charge_id}\n"
            f")"
        )

class CopyTextButton:
    def __init__(self, text: str):
        self.text = text

    def to_dict(self):
        return {
            "text": self.text,
            "callback_data": f"copy:{self.text}"
        }

class ChatPhoto:
    def __init__(self, small_file_id: str, big_file_id: str):
        self.small_file_id = small_file_id
        self.big_file_id = big_file_id

    def to_dict(self):
        return {
            "small_file_id": self.small_file_id,
            "big_file_id": self.big_file_id
        }

class ChatParameter:
    def __init__(self, chat_data: dict):
        self.id = chat_data.get("id")
        self.type = chat_data.get("type")
        self.title = chat_data.get("title")
        self.username = chat_data.get("username")
        self.photo = chat_data.get("photo")
        self.description = chat_data.get("description")
        self.invite_link = chat_data.get("invite_link")
        self.permissions = chat_data.get("permissions")

    def __str__(self):
        return (
            f"ChatParameter(\n"
            f"    id={self.id},\n"
            f"    type={self.type},\n"
            f"    title={self.title},\n"
            f"    username={self.username},\n"
            f"    photo={self.photo},\n"
            f"    description={self.description},\n"
            f"    invite_link={self.invite_link},\n"
            f"    permissions={self.permissions}\n"
            f")"
        )

class InputMedia:
    def __init__(self, type: str, media: str, caption: str = None, parse_mode: str = None):
        self.type = type
        self.media = media
        self.caption = caption
        self.parse_mode = parse_mode

    def to_dict(self):
        data = {
            "type": self.type,
            "media": self.media
        }
        if self.caption:
            data["caption"] = self.caption
        if self.parse_mode:
            data["parse_mode"] = self.parse_mode
        return data

class InputMediaPhoto(InputMedia):
    def __init__(self, media: str, caption: str = None, parse_mode: str = None):
        super().__init__("photo", media, caption, parse_mode)

class InputMediaVideo(InputMedia):
    def __init__(self, media: str, caption: str = None, parse_mode: str = None, width: int = None, height: int = None, duration: int = None):
        super().__init__("video", media, caption, parse_mode)
        self.width = width
        self.height = height
        self.duration = duration

    def to_dict(self):
        data = super().to_dict()
        if self.width:
            data["width"] = self.width
        if self.height:
            data["height"] = self.height
        if self.duration:
            data["duration"] = self.duration
        return data

class InputMediaAnimation(InputMedia):
    def __init__(self, media: str, caption: str = None, parse_mode: str = None, duration: int = None, width: int = None, height: int = None):
        super().__init__("animation", media, caption, parse_mode)
        self.duration = duration
        self.width = width
        self.height = height

    def to_dict(self):
        data = super().to_dict()
        if self.duration:
            data["duration"] = self.duration
        if self.width:
            data["width"] = self.width
        if self.height:
            data["height"] = self.height
        return data

class InputMediaAudio(InputMedia):
    def __init__(self, media: str, caption: str = None, parse_mode: str = None, duration: int = None, performer: str = None, title: str = None):
        super().__init__("audio", media, caption, parse_mode)
        self.duration = duration
        self.performer = performer
        self.title = title

    def to_dict(self):
        data = super().to_dict()
        if self.duration:
            data["duration"] = self.duration
        if self.performer:
            data["performer"] = self.performer
        if self.title:
            data["title"] = self.title
        return data

class InputMediaDocument(InputMedia):
    def __init__(self, media: str, caption: str = None, parse_mode: str = None, disable_content_type_detection: bool = None):
        super().__init__("document", media, caption, parse_mode)
        self.disable_content_type_detection = disable_content_type_detection

    def to_dict(self):
        data = super().to_dict()
        if self.disable_content_type_detection is not None:
            data["disable_content_type_detection"] = self.disable_content_type_detection
        return data

class InputFile:
    def __init__(self, file_path: str, file_name: str = None, mime_type: str = None):
        self.file_path = file_path
        self.file_name = file_name
        self.mime_type = mime_type

    def to_dict(self):
        data = {
            "file_path": self.file_path
        }
        if self.file_name:
            data["file_name"] = self.file_name
        if self.mime_type:
            data["mime_type"] = self.mime_type
        return data

class Bot:
    def __init__(self, Token: str, url: str = None, concurrency_limit: int = None, proxy: str = None):
        self.token = Token
        self.base_url = url if url is not None else "https://tapi.bale.ai"
        self.handlers: List[Dict] = []
        self.callback_handlers: List[Dict] = []
        self.running = asyncio.Event()
        self.user_states: Dict[str, Dict[int, str]] = {}
        self.user_data = {}
        self.filters = Filters(self)
        self.initialize_handlers: List[Callable] = []
        self.concurrency_limit = concurrency_limit
        self.active_tasks = set()
        self.proxy = proxy

    class ChatParameter:
        def __init__(self, chat_data: dict):
            self.id = chat_data.get("id")
            self.type = chat_data.get("type")
            self.title = chat_data.get("title")
            self.username = chat_data.get("username")
            self.photo = chat_data.get("photo")
            self.description = chat_data.get("description")
            self.invite_link = chat_data.get("invite_link")
            self.permissions = chat_data.get("permissions")

        def __str__(self):
            return (
                f"ChatParameter(\n"
                f"    id={self.id},\n"
                f"    type={self.type},\n"
                f"    title={self.title},\n"
                f"    username={self.username},\n"
                f"    photo={self.photo},\n"
                f"    description={self.description},\n"
                f"    invite_link={self.invite_link},\n"
                f"    permissions={self.permissions}\n"
                f")"
            )

    async def get_chat(self, chat_id: int):
        url = f"{self.base_url}/bot{self.token}/getChat"
        params = {"chat_id": chat_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()
                if response_data.get("ok"):
                    return self.ChatParameter(response_data["result"])
                else:
                    return None

    def set_user_state(self, user_id: int, state: str):
        if self.token not in self.user_states:
            self.user_states[self.token] = {}
        self.user_states[self.token][user_id] = state

    def get_user_state(self, user_id: int) -> str:
        if self.token in self.user_states and user_id in self.user_states[self.token]:
            return self.user_states[self.token][user_id]
        return None

    def clear_user_state(self, user_id: int):
        if self.token in self.user_states and user_id in self.user_states[self.token]:
            del self.user_states[self.token][user_id]

    def set_user_data(self, user_id: str, key: str, value: any):
        if user_id not in self.user_data:
            self.user_data[user_id] = {}
        self.user_data[user_id][key] = value

    def get_user_data(self, user_id: str, key: str) -> Optional[any]:
        return self.user_data.get(user_id, {}).get(key)

    def Message(self, _filter: Filters = None):
        if _filter is None:
            _filter = self.filters.any_message()

        def decorator(func: Callable):
            self.handlers.append({"filter": _filter, "func": func})
            return func
        return decorator

    async def get_me(self):
        url = f"{self.base_url}/bot{self.token}/getMe"
    
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                return await response.json()

    async def get_updates(self, offset=None, timeout=30):
        url = f"{self.base_url}/bot{self.token}/getUpdates"
        params = {"timeout": timeout}
        if offset is not None:
            params["offset"] = offset

        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url, params=params, proxy=self.proxy) as response:
                    if response.status != 200:
                        print(f"HTTP Error: {response.status}")
                        return None

                    response_data = await response.json()

                    if response_data is None:
                        print("Empty response from server")
                        return None

                    if not isinstance(response_data, dict) or "ok" not in response_data:
                        print("Invalid response format")
                        return None

                    if not response_data.get("ok"):
                        print(f"API Error: {response_data.get('description', 'Unknown error')}")
                        return None

                    return response_data.get("result", [])
        except Exception as e:
            print(f"An error occurred in get_updates: {e}")
            return None

    async def process_updates(self):
        offset = None
        while self.running.is_set():
            try:
                updates = await self.get_updates(offset=offset)
                if updates is None:
                    print("No updates received or invalid response.")
                    continue
                for update in updates:
                    offset = update["update_id"] + 1
                    update_wrapper = UpdateWrapper(update)
                    if self.concurrency_limit is not None and len(self.active_tasks) >= self.concurrency_limit:
                        print("Concurrency limit reached, skipping update.")
                        continue
                    task = asyncio.create_task(self._process_update(update_wrapper))
                    self.active_tasks.add(task)
                    task.add_done_callback(self.active_tasks.discard)
            except Exception as e:
                print(f"An error occurred in process_updates: {e}")

    async def _process_update(self, update_wrapper):
            try:
                if update_wrapper.callback_query:
                    callback_data = update_wrapper.callback_query.data

                    for handler in self.callback_handlers:
                        if handler["filter"](update_wrapper.update):
                            result = await handler["func"](
                                self, 
                                update_wrapper.update,
                                update_wrapper.callback_query
                            )
                            if result and not result.get("ok"):
                                print(f"Handler failed: {result.get('description')}")
                            return

                if update_wrapper.message:
                    for handler in self.handlers:
                        if handler["filter"](update_wrapper.update):
                            result = await handler["func"](
                                self, 
                                update_wrapper.update, 
                                update_wrapper.message
                            )
                            if result and not result.get("ok"):
                                print(f"Handler failed: {result.get('description')}")
                            return
            except Exception as e:
                print(f"Error in _process_update: {str(e)}")

    def Initialize(self):
        def decorator(func: Callable):
            self.initialize_handlers.append(func)
            return func
        return decorator

    async def run_initialize_handlers(self):
        for handler in self.initialize_handlers:
            await handler(self)

    async def start(self):
        self.running.set()
        try:
            bot_info = await self.get_me()
            if bot_info.get("ok"):
                print(f"Bot is running! Username: @{bot_info['result']['username']}")
            else:
                print("Failed to start the bot. Please check your token and API URL.")
                return
        except Exception as e:
            print(f"An error occurred while starting the bot: {e}")
            return

        await self.run_initialize_handlers()
        await self.process_updates()

    async def stop(self):
        self.running.clear()
        print("Bot has been stopped.")

    async def delete_message_auto(self, message: dict):
        chat_id = message["chat"]["id"]
        message_id = message["message_id"]
        return await self.delete_message(chat_id=chat_id, message_id=message_id)

    async def edit_message_text_auto(self, message: dict, text: str, reply_markup=None):
        chat_id = message["chat"]["id"]
        message_id = message["message_id"]
        return await self.edit_message_text(chat_id=chat_id, message_id=message_id, text=text, reply_markup=reply_markup)

    async def edit_message_caption_auto(
        self,
        chat_id: int,
        message_id: int,
        caption: str,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/editMessageCaption"
        params = {"chat_id": chat_id, "message_id": message_id, "caption": caption}
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def schedule_message(
        self,
        chat_id: int,
        text: str,
        delay_seconds: int,
        reply_to_message_id: int = None,
        reply_markup=None,
        edit_message_id: int = None,
    ):
        await asyncio.sleep(delay_seconds)
        return await self.send_message(
            chat_id=chat_id,
            text=text,
            reply_to_message_id=reply_to_message_id,
            reply_markup=reply_markup,
            edit_message_id=edit_message_id,
        )

    async def send_message(
        self,
        chat_id: int,
        text: str,
        reply_to_message_id: Optional[int] = None,
        reply_markup: Optional[Dict[str, Any]] = None,
        edit_message_id: Optional[int] = None,
    ) -> Dict[str, Any]:
        params = {"chat_id": chat_id, "text": text}
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict() if hasattr(reply_markup, 'to_dict') else reply_markup

        url = f"{self.base_url}/bot{self.token}/sendMessage"

        try:
            async with aiohttp.ClientSession() as session:
                async with session.post(
                    url,
                    json=params,
                    proxy=self.proxy,
                    headers={'Content-Type': 'application/json'}
                ) as response:
                    # ابتدا پاسخ را به صورت متنی دریافت کنید
                    response_text = await response.text()
                    try:
                        response_data = await response.json()
                    except:
                        print(f"Failed to decode JSON. Raw response: {response_text}")
                        response_data = {"ok": False, "description": response_text}

                    if not response_data.get("ok"):
                        print(f"API Error: {response_data.get('description')}")

                    return response_data
        except Exception as e:
            print(f"Network error in send_message: {str(e)}")
            return {"ok": False, "error": str(e)}

    async def pin_chat_message(
        self,
        chat_id: int,
        message_id: int,
        disable_notification: bool = False
    ):
        url = f"{self.base_url}/bot{self.token}/pinChatMessage"
        params = {
            "chat_id": chat_id,
            "message_id": message_id,
            "disable_notification": disable_notification
        }
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()
                return response_data.get("ok", False)

    async def unpin_chat_message(
        self,
        chat_id: int,
        message_id: int = None
    ):
        url = f"{self.base_url}/bot{self.token}/unpinChatMessage"
        params = {"chat_id": chat_id}
        if message_id is not None:
            params["message_id"] = message_id
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()
                return response_data.get("ok", False)

    async def unpin_all_chat_messages(
        self,
        chat_id: int
    ):
        url = f"{self.base_url}/bot{self.token}/unpinAllChatMessages"
        params = {"chat_id": chat_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()
                return response_data.get("ok", False)

    async def pin_message_auto(
        self,
        message: dict,
        disable_notification: bool = False
    ):
        chat_id = message["chat"]["id"]
        message_id = message["message_id"]
        return await self.pin_chat_message(
            chat_id=chat_id,
            message_id=message_id,
            disable_notification=disable_notification
        )

    async def unpin_message_auto(
        self,
        message: dict
    ):
        chat_id = message["chat"]["id"]
        message_id = message["message_id"]
        return await self.unpin_chat_message(
            chat_id=chat_id,
            message_id=message_id
        )

    async def send_animation(
        self,
        chat_id: int,
        animation: str,
        caption: str = None,
        reply_to_message_id: int = None,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/sendAnimation"
        params = {"chat_id": chat_id, "animation": animation}
        if caption:
            params["caption"] = caption
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def send_audio(
        self,
        chat_id: int,
        audio: str,
        caption: str = None,
        reply_to_message_id: int = None,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/sendAudio"
        params = {"chat_id": chat_id, "audio": audio}
        if caption:
            params["caption"] = caption
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def send_contact(
        self,
        chat_id: int,
        phone_number: str,
        first_name: str,
        last_name: str = None,
        reply_to_message_id: int = None,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/sendContact"
        params = {"chat_id": chat_id, "phone_number": phone_number, "first_name": first_name}
        if last_name:
            params["last_name"] = last_name
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def send_document(
        self,
        chat_id: int,
        document: str,
        caption: str = None,
        reply_to_message_id: int = None,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/sendDocument"
        params = {"chat_id": chat_id, "document": document}
        if caption:
            params["caption"] = caption
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def send_location(
        self,
        chat_id: int,
        latitude: float,
        longitude: float,
        reply_to_message_id: int = None,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/sendLocation"
        params = {"chat_id": chat_id, "latitude": latitude, "longitude": longitude}
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def send_media_group(
        self,
        chat_id: int,
        media: List[Union[InputMediaPhoto, InputMediaVideo, InputMediaAnimation, InputMediaAudio, InputMediaDocument]],
        reply_to_message_id: int = None,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/sendMediaGroup"
        params = {
            "chat_id": chat_id,
            "media": [m.to_dict() for m in media]
        }
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()

        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                return await response.json()

    async def send_input_file(
        self,
        chat_id: int,
        input_file: InputFile,
        caption: str = None,
        reply_to_message_id: int = None,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/sendDocument"
        params = {
            "chat_id": chat_id,
            "document": input_file.to_dict()
        }
        if caption:
            params["caption"] = caption
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()

        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                return await response.json()

    async def send_photo(
        self,
        chat_id: int,
        photo: str,
        caption: str = None,
        reply_to_message_id: int = None,
        reply_markup=None,
        edit_message_id: int = None,
    ):
        if edit_message_id:
            return await self.edit_message_caption(chat_id=chat_id, message_id=edit_message_id, caption=caption, reply_markup=reply_markup)
        else:
            url = f"{self.base_url}/bot{self.token}/sendPhoto"
            params = {"chat_id": chat_id, "photo": photo}
            if caption:
                params["caption"] = caption
            if reply_to_message_id:
                params["reply_to_message_id"] = reply_to_message_id
            if reply_markup:
                params["reply_markup"] = reply_markup.to_dict()
            async with aiohttp.ClientSession() as session:
                async with session.post(url, json=params, proxy=self.proxy) as response:
                    response_data = await response.json()

    async def send_video(
        self,
        chat_id: int,
        video: str,
        caption: str = None,
        reply_to_message_id: int = None,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/sendVideo"
        params = {"chat_id": chat_id, "video": video}
        if caption:
            params["caption"] = caption
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def send_voice(
        self,
        chat_id: int,
        voice: str,
        caption: str = None,
        reply_to_message_id: int = None,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/sendVoice"
        params = {"chat_id": chat_id, "voice": voice}
        if caption:
            params["caption"] = caption
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def send_sticker(
        self,
        chat_id: int,
        sticker: str,
        reply_to_message_id: int = None,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/sendSticker"
        params = {"chat_id": chat_id, "sticker": sticker}
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def send_chat_action(self, chat_id: int, action: str):
        url = f"{self.base_url}/bot{self.token}/sendChatAction"
        params = {"chat_id": chat_id, "action": action}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def edit_message_text(
        self,
        chat_id: int,
        message_id: int,
        text: str,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/editMessageText"
        params = {"chat_id": chat_id, "message_id": message_id, "text": text}
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def delete_message(self, chat_id: int, message_id: int):
        url = f"{self.base_url}/bot{self.token}/deleteMessage"
        params = {"chat_id": chat_id, "message_id": message_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def forward_message(
        self,
        chat_id: int,
        from_chat_id: int,
        message_id: int,
    ):
        url = f"{self.base_url}/bot{self.token}/forwardMessage"
        params = {"chat_id": chat_id, "from_chat_id": from_chat_id, "message_id": message_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def get_chat_administrators(self, chat_id: str):
        url = f"{self.base_url}/bot{self.token}/getChatAdministrators"
        params = {"chat_id": chat_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()
                return response_data

    async def get_chat_member(self, chat_id: int, user_id: int):
        url = f"{self.base_url}/bot{self.token}/getChatMember"
        params = {"chat_id": chat_id, "user_id": user_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()
                return response_data

    async def get_chat_members_count(self, chat_id: int):
        url = f"{self.base_url}/bot{self.token}/getChatMembersCount"
        params = {"chat_id": chat_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()
                return response_data

    async def get_file(self, file_id: str):
        url = f"{self.base_url}/bot{self.token}/getFile"
        params = {"file_id": file_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()
                return response_data

    async def get_sticker_set(self, name: str):
        url = f"{self.base_url}/bot{self.token}/getStickerSet"
        params = {"name": name}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()
                return response_data

    async def invite_user(self, chat_id: int, user_id: int):
        url = f"{self.base_url}/bot{self.token}/inviteUser"
        params = {"chat_id": chat_id, "user_id": user_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def leave_chat(self, chat_id: int):
        url = f"{self.base_url}/bot{self.token}/leaveChat"
        params = {"chat_id": chat_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def promote_chat_member(
        self,
        chat_id: int,
        user_id: int,
        can_change_info: bool = None,
        can_post_messages: bool = None,
        can_edit_messages: bool = None,
        can_delete_messages: bool = None,
        can_invite_users: bool = None,
        can_restrict_members: bool = None,
        can_pin_messages: bool = None,
        can_promote_members: bool = None,
    ):
        url = f"{self.base_url}/bot{self.token}/promoteChatMember"
        params = {"chat_id": chat_id, "user_id": user_id}
        if can_change_info is not None:
            params["can_change_info"] = can_change_info
        if can_post_messages is not None:
            params["can_post_messages"] = can_post_messages
        if can_edit_messages is not None:
            params["can_edit_messages"] = can_edit_messages
        if can_delete_messages is not None:
            params["can_delete_messages"] = can_delete_messages
        if can_invite_users is not None:
            params["can_invite_users"] = can_invite_users
        if can_restrict_members is not None:
            params["can_restrict_members"] = can_restrict_members
        if can_pin_messages is not None:
            params["can_pin_messages"] = can_pin_messages
        if can_promote_members is not None:
            params["can_promote_members"] = can_promote_members
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def set_chat_photo(self, chat_id: int, photo: str):
        url = f"{self.base_url}/bot{self.token}/setChatPhoto"
        params = {"chat_id": chat_id, "photo": photo}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def ban_chat_member(self, chat_id: int, user_id: int):
        url = f"{self.base_url}/bot{self.token}/banChatMember"
        params = {"chat_id": chat_id, "user_id": user_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def unban_chat_member(self, chat_id: int, user_id: int):
        url = f"{self.base_url}/bot{self.token}/unbanChatMember"
        params = {"chat_id": chat_id, "user_id": user_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def copy_message(
        self,
        chat_id: int,
        from_chat_id: int,
        message_id: int,
        caption: str = None,
        reply_to_message_id: int = None,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/copyMessage"
        params = {"chat_id": chat_id, "from_chat_id": from_chat_id, "message_id": message_id}
        if caption:
            params["caption"] = caption
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def add_sticker_to_set(
        self,
        user_id: int,
        name: str,
        png_sticker: str,
        emojis: str,
        mask_position: dict = None,
    ):
        url = f"{self.base_url}/bot{self.token}/addStickerToSet"
        params = {"user_id": user_id, "name": name, "png_sticker": png_sticker, "emojis": emojis}
        if mask_position:
            params["mask_position"] = mask_position
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def create_new_sticker_set(
        self,
        user_id: int,
        name: str,
        title: str,
        png_sticker: str,
        emojis: str,
        contains_masks: bool = None,
        mask_position: dict = None,
    ):
        url = f"{self.base_url}/bot{self.token}/createNewStickerSet"
        params = {"user_id": user_id, "name": name, "title": title, "png_sticker": png_sticker, "emojis": emojis}
        if contains_masks is not None:
            params["contains_masks"] = contains_masks
        if mask_position:
            params["mask_position"] = mask_position
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def upload_sticker_file(self, user_id: int, png_sticker: str):
        url = f"{self.base_url}/bot{self.token}/uploadStickerFile"
        params = {"user_id": user_id, "png_sticker": png_sticker}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def create_chat_invite_link(
        self,
        chat_id: int,
        expire_date: int = None,
        member_limit: int = None,
    ):
        url = f"{self.base_url}/bot{self.token}/createChatInviteLink"
        params = {"chat_id": chat_id}
        if expire_date:
            params["expire_date"] = expire_date
        if member_limit:
            params["member_limit"] = member_limit
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def delete_chat_photo(self, chat_id: int):
        url = f"{self.base_url}/bot{self.token}/deleteChatPhoto"
        params = {"chat_id": chat_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def delete_sticker_from_set(self, sticker: str):
        url = f"{self.base_url}/bot{self.token}/deleteStickerFromSet"
        params = {"sticker": sticker}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def edit_message_caption(
        self,
        chat_id: int,
        message_id: int,
        caption: str,
        reply_markup=None,
    ):
        url = f"{self.base_url}/bot{self.token}/editMessageCaption"
        params = {"chat_id": chat_id, "message_id": message_id, "caption": caption}
        if reply_markup:
            params["reply_markup"] = reply_markup.to_dict()
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def export_chat_invite_link(self, chat_id: int):
        url = f"{self.base_url}/bot{self.token}/exportChatInviteLink"
        params = {"chat_id": chat_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def pin_chat_message(
        self,
        chat_id: int,
        message_id: int,
        disable_notification: bool = None,
    ):
        url = f"{self.base_url}/bot{self.token}/pinChatMessage"
        params = {"chat_id": chat_id, "message_id": message_id}
        if disable_notification is not None:
            params["disable_notification"] = disable_notification
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def revoke_chat_invite_link(self, chat_id: int, invite_link: str):
        url = f"{self.base_url}/bot{self.token}/revokeChatInviteLink"
        params = {"chat_id": chat_id, "invite_link": invite_link}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def set_chat_description(self, chat_id: int, description: str):
        url = f"{self.base_url}/bot{self.token}/setChatDescription"
        params = {"chat_id": chat_id, "description": description}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def set_chat_title(self, chat_id: int, title: str):
        url = f"{self.base_url}/bot{self.token}/setChatTitle"
        params = {"chat_id": chat_id, "title": title}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def unpin_all_chat_messages(self, chat_id: int):
        url = f"{self.base_url}/bot{self.token}/unpinAllChatMessages"
        params = {"chat_id": chat_id}
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def unpin_chat_message(self, chat_id: int, message_id: int = None):
        url = f"{self.base_url}/bot{self.token}/unpinChatMessage"
        params = {"chat_id": chat_id}
        if message_id:
            params["message_id"] = message_id
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def reply_message_auto(self, update: dict, text: str, reply_markup=None):
        update_wrapper = UpdateWrapper(update)
        if update_wrapper.message:
            chat_id = update_wrapper.message.chat.id
            message_id = update_wrapper.message.message_id
            return await self.send_message(
                chat_id=chat_id,
                text=text,
                reply_to_message_id=message_id,
                reply_markup=reply_markup,
            )
        return {"ok": False, "error": "Invalid update format"}

    async def reply_video_auto(self, update: dict, video: str, caption: str = None, reply_markup=None):
        update_wrapper = UpdateWrapper(update)
        if update_wrapper.message:
            return await self.send_video(
                chat_id=update_wrapper.message.chat.id,
                video=video,
                caption=caption,
                reply_to_message_id=update_wrapper.message.message_id,
                reply_markup=reply_markup,
            )
        return {"ok": False, "error": "Invalid update format"}

    async def reply_voice_auto(self, update: dict, voice: str, caption: str = None, reply_markup=None):
        update_wrapper = UpdateWrapper(update)
        if update_wrapper.message:
            chat_id = update_wrapper.message.chat.id
            message_id = update_wrapper.message.message_id
            return await self.send_voice(
                chat_id=chat_id,
                voice=voice,
                caption=caption,
                reply_to_message_id=message_id,
                reply_markup=reply_markup,
            )
        return {"ok": False, "error": "Invalid update format"}

    async def reply_sticker_auto(self, update: dict, sticker: str, reply_markup=None):
        update_wrapper = UpdateWrapper(update)
        if update_wrapper.message:
            chat_id = update_wrapper.message.chat.id
            message_id = update_wrapper.message.message_id
            return await self.send_sticker(
                chat_id=chat_id,
                sticker=sticker,
                reply_to_message_id=message_id,
                reply_markup=reply_markup,
            )
        return {"ok": False, "error": "Invalid update format"}

    async def reply_document_auto(self, update: dict, document: str, caption: str = None, reply_markup=None):
        update_wrapper = UpdateWrapper(update)
        if update_wrapper.message:
            chat_id = update_wrapper.message.chat.id
            message_id = update_wrapper.message.message_id
            return await self.send_document(
                chat_id=chat_id,
                document=document,
                caption=caption,
                reply_to_message_id=message_id,
                reply_markup=reply_markup,
            )
        return {"ok": False, "error": "Invalid update format"}

    async def reply_animation_auto(self, update: dict, animation: str, caption: str = None, reply_markup=None):
        update_wrapper = UpdateWrapper(update)
        if update_wrapper.message:
            chat_id = update_wrapper.message.chat.id
            message_id = update_wrapper.message.message_id
            return await self.send_animation(
                chat_id=chat_id,
                animation=animation,
                caption=caption,
                reply_to_message_id=message_id,
                reply_markup=reply_markup,
            )
        return {"ok": False, "error": "Invalid update format"}

    async def reply_audio_auto(self, update: dict, audio: str, caption: str = None, reply_markup=None):
        update_wrapper = UpdateWrapper(update)
        if update_wrapper.message:
            chat_id = update_wrapper.message.chat.id
            message_id = update_wrapper.message.message_id
            return await self.send_audio(
                chat_id=chat_id,
                audio=audio,
                caption=caption,
                reply_to_message_id=message_id,
                reply_markup=reply_markup,
            )
        return {"ok": False, "error": "Invalid update format"}

    async def reply_photo_auto(self, update: dict, photo: str, caption: str = None, reply_markup=None):
        update_wrapper = UpdateWrapper(update)
        if update_wrapper.message:
            chat_id = update_wrapper.message.chat.id
            message_id = update_wrapper.message.message_id
            return await self.send_photo(
                chat_id=chat_id,
                photo=photo,
                caption=caption,
                reply_to_message_id=message_id,
                reply_markup=reply_markup,
            )
        return {"ok": False, "error": "Invalid update format"}

    async def reply_message(self, chat_id: int, text: str, message_id: int, reply_markup=None):
        return await self.send_message(
            chat_id=chat_id,
            text=text,
            reply_to_message_id=message_id,
            reply_markup=reply_markup,
        )

    async def reply_video(self, chat_id: int, video: str, message_id: int, caption: str = None, reply_markup=None):
        return await self.send_video(
            chat_id=chat_id,
            video=video,
            caption=caption,
            reply_to_message_id=message_id,
            reply_markup=reply_markup,
        )

    async def reply_voice(self, chat_id: int, voice: str, message_id: int, caption: str = None, reply_markup=None):
        return await self.send_voice(
            chat_id=chat_id,
            voice=voice,
            caption=caption,
            reply_to_message_id=message_id,
            reply_markup=reply_markup,
        )

    async def reply_sticker(self, chat_id: int, sticker: str, message_id: int, reply_markup=None):
        return await self.send_sticker(
            chat_id=chat_id,
            sticker=sticker,
            reply_to_message_id=message_id,
            reply_markup=reply_markup,
        )

    async def reply_document(self, chat_id: int, document: str, message_id: int, caption: str = None, reply_markup=None):
        return await self.send_document(
            chat_id=chat_id,
            document=document,
            caption=caption,
            reply_to_message_id=message_id,
            reply_markup=reply_markup,
        )

    async def reply_animation(self, chat_id: int, animation: str, message_id: int, caption: str = None, reply_markup=None):
        return await self.send_animation(
            chat_id=chat_id,
            animation=animation,
            caption=caption,
            reply_to_message_id=message_id,
            reply_markup=reply_markup,
        )

    async def reply_audio(self, chat_id: int, audio: str, message_id: int, caption: str = None, reply_markup=None):
        return await self.send_audio(
            chat_id=chat_id,
            audio=audio,
            caption=caption,
            reply_to_message_id=message_id,
            reply_markup=reply_markup,
        )

    async def reply_photo(self, chat_id: int, photo: str, message_id: int, caption: str = None, reply_markup=None):
        return await self.send_photo(
            chat_id=chat_id,
            photo=photo,
            caption=caption,
            reply_to_message_id=message_id,
            reply_markup=reply_markup,
        )

    def CallbackQuery(self, _filter: Filters = None):
        if _filter is None:
            _filter = self.filters.callback_query_all()

        def decorator(func: Callable):
            self.callback_handlers.append({
                "filter": _filter, 
                "func": func
            })
            return func
        return decorator

    def LabeledPrice(label: str, amount: int):
        def decorator(func):
            def wrapper(*args, **kwargs):
                prices = [
                    {"label": label, "amount": amount},
                ]
                return func(*args, prices=prices, **kwargs)
            return wrapper
        return decorator


    def PreCheckoutQuery(self):
        def decorator(func: Callable):
            self.callback_handlers.append({"filter": self.filters.pre_checkout_query(), "func": func})
            return func
        return decorator


    class SuccessfulPayment:
        def __init__(self, successful_payment_data: dict):
            self.currency = successful_payment_data.get("currency")
            self.total_amount = successful_payment_data.get("total_amount")
            self.invoice_payload = successful_payment_data.get("invoice_payload")
            self.telegram_payment_charge_id = successful_payment_data.get("telegram_payment_charge_id")

    async def send_invoice(
        self,
        chat_id: int,
        title: str,
        description: str,
        payload: str,
        provider_token: str,
        prices: list,
        photo_url: str = None,
        reply_to_message_id: int = None,
    ):
        url = f"{self.base_url}/bot{self.token}/sendInvoice"
        params = {
            "chat_id": chat_id,
            "title": title,
            "description": description,
            "payload": payload,
            "provider_token": provider_token,
            "prices": prices,
        }

        if photo_url:
            params["photo_url"] = photo_url
        if reply_to_message_id:
            params["reply_to_message_id"] = reply_to_message_id

        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

    async def answer_pre_checkout_query(
        self,
        pre_checkout_query_id: str,
        ok: bool,
        error_message: str = None,
    ):
        url = f"{self.base_url}/bot{self.token}/answerPreCheckoutQuery"
        params = {
            "pre_checkout_query_id": pre_checkout_query_id,
            "ok": ok,
        }
        if error_message:
            params["error_message"] = error_message

        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=params, proxy=self.proxy) as response:
                response_data = await response.json()

class Chat:
    def __init__(self, chat_data: dict):
        self.id = chat_data.get("id")
        self.type = chat_data.get("type")


class Message:
    def __init__(self, message_data: dict):
        self.chat = Chat(message_data.get("chat", {}))
        self.message_id = message_data.get("message_id")
        self.text = message_data.get("text")
        self.data = message_data
        self.bot = None

    async def reply(self, text: str, reply_markup=None, **kwargs):
        """Reply to this message with text"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.send_message(
            chat_id=self.chat.id,
            text=text,
            reply_to_message_id=self.message_id,
            reply_markup=reply_markup,
            **kwargs
        )

    async def reply_text(self, text: str, reply_markup=None, **kwargs):
        """Alias for reply()"""
        return await self.reply(text, reply_markup, **kwargs)

    async def copy(self, chat_id: int, reply_markup=None, **kwargs):
        """Copy this message to another chat"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.copy_message(
            chat_id=chat_id,
            from_chat_id=self.chat.id,
            message_id=self.message_id,
            reply_markup=reply_markup,
            **kwargs
        )

    async def forward(self, chat_id: int, **kwargs):
        """Forward this message to another chat"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.forward_message(
            chat_id=chat_id,
            from_chat_id=self.chat.id,
            message_id=self.message_id,
            **kwargs
        )

    async def reply_video(self, video: str, caption: str = None, reply_markup=None, **kwargs):
        """Reply to this message with a video"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.send_video(
            chat_id=self.chat.id,
            video=video,
            caption=caption,
            reply_to_message_id=self.message_id,
            reply_markup=reply_markup,
            **kwargs
        )

    async def reply_photo(self, photo: str, caption: str = None, reply_markup=None, **kwargs):
        """Reply to this message with a photo"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.send_photo(
            chat_id=self.chat.id,
            photo=photo,
            caption=caption,
            reply_to_message_id=self.message_id,
            reply_markup=reply_markup,
            **kwargs
        )

    async def reply_document(self, document: str, caption: str = None, reply_markup=None, **kwargs):
        """Reply to this message with a document"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.send_document(
            chat_id=self.chat.id,
            document=document,
            caption=caption,
            reply_to_message_id=self.message_id,
            reply_markup=reply_markup,
            **kwargs
        )

    async def reply_voice(self, voice: str, caption: str = None, reply_markup=None, **kwargs):
        """Reply to this message with a voice message"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.send_voice(
            chat_id=self.chat.id,
            voice=voice,
            caption=caption,
            reply_to_message_id=self.message_id,
            reply_markup=reply_markup,
            **kwargs
        )

    async def reply_audio(self, audio: str, caption: str = None, reply_markup=None, **kwargs):
        """Reply to this message with an audio file"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.send_audio(
            chat_id=self.chat.id,
            audio=audio,
            caption=caption,
            reply_to_message_id=self.message_id,
            reply_markup=reply_markup,
            **kwargs
        )

    async def reply_animation(self, animation: str, caption: str = None, reply_markup=None, **kwargs):
        """Reply to this message with an animation"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.send_animation(
            chat_id=self.chat.id,
            animation=animation,
            caption=caption,
            reply_to_message_id=self.message_id,
            reply_markup=reply_markup,
            **kwargs
        )

    async def reply_sticker(self, sticker: str, reply_markup=None, **kwargs):
        """Reply to this message with a sticker"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.send_sticker(
            chat_id=self.chat.id,
            sticker=sticker,
            reply_to_message_id=self.message_id,
            reply_markup=reply_markup,
            **kwargs
        )

    async def reply_contact(self, phone_number: str, first_name: str, last_name: str = None, reply_markup=None, **kwargs):
        """Reply to this message with a contact"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.send_contact(
            chat_id=self.chat.id,
            phone_number=phone_number,
            first_name=first_name,
            last_name=last_name,
            reply_to_message_id=self.message_id,
            reply_markup=reply_markup,
            **kwargs
        )

    async def reply_location(self, latitude: float, longitude: float, reply_markup=None, **kwargs):
        """Reply to this message with a location"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.send_location(
            chat_id=self.chat.id,
            latitude=latitude,
            longitude=longitude,
            reply_to_message_id=self.message_id,
            reply_markup=reply_markup,
            **kwargs
        )

    async def delete(self, **kwargs):
        """Delete this message"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.delete_message(
            chat_id=self.chat.id,
            message_id=self.message_id,
            **kwargs
        )

    async def edit_text(self, text: str, reply_markup=None, **kwargs):
        """Edit this message's text"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.edit_message_text(
            chat_id=self.chat.id,
            message_id=self.message_id,
            text=text,
            reply_markup=reply_markup,
            **kwargs
        )

    async def edit_caption(self, caption: str, reply_markup=None, **kwargs):
        """Edit this message's caption"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.edit_message_caption(
            chat_id=self.chat.id,
            message_id=self.message_id,
            caption=caption,
            reply_markup=reply_markup,
            **kwargs
        )

    async def pin(self, disable_notification: bool = False, **kwargs):
        """Pin this message in the chat"""
        if not self.bot:
            raise ValueError("Bot instance not set on this Message object")
        return await self.bot.pin_chat_message(
            chat_id=self.chat.id,
            message_id=self.message_id,
            disable_notification=disable_notification,
            **kwargs
        )


from typing import Dict, List, Optional, Any, Union
from threading import Lock
import asyncio
import time

class State:
    def __init__(self, name: str, default_data: Optional[Dict] = None):
        self.name = name
        self.default_data = default_data or {}
        self._user_states: Dict[int, Dict] = {}
        self._lock = Lock()
    
    def set_state(self, user_id: int, data: Optional[Dict] = None) -> None:
        with self._lock:
            self._user_states[user_id] = {**self.default_data, **(data or {})}
    
    def get_state(self, user_id: int) -> Optional[Dict]:
        with self._lock:
            return self._user_states.get(user_id)
    
    def update_state(self, user_id: int, updates: Dict) -> bool:
        with self._lock:
            if user_id not in self._user_states:
                return False
            self._user_states[user_id].update(updates)
            return True
    
    def clear_state(self, user_id: int) -> bool:
        with self._lock:
            if user_id in self._user_states:
                del self._user_states[user_id]
                return True
            return False
    
    def has_state(self, user_id: int) -> bool:
        with self._lock:
            return user_id in self._user_states


class TimedState(State):
    def __init__(self, name: str, default_data: Optional[Dict] = None, timeout: int = 3600):
        super().__init__(name, default_data)
        self._timeouts: Dict[int, float] = {}
        self.timeout = timeout
    
    def set_state(self, user_id: int, data: Optional[Dict] = None) -> None:
        super().set_state(user_id, data)
        with self._lock:
            self._timeouts[user_id] = time.time() + self.timeout
    
    def get_state(self, user_id: int) -> Optional[Dict]:
        with self._lock:
            if user_id in self._timeouts and time.time() > self._timeouts[user_id]:
                del self._timeouts[user_id]
                if user_id in self._user_states:
                    del self._user_states[user_id]
                return None
        return super().get_state(user_id)
    
    def clear_state(self, user_id: int) -> bool:
        with self._lock:
            if user_id in self._timeouts:
                del self._timeouts[user_id]
        return super().clear_state(user_id)


class ChatState:
    def __init__(self, name: str, default_data: Optional[Dict] = None):
        self.name = name
        self.default_data = default_data or {}
        self._chat_states: Dict[int, Dict] = {}
        self._user_states_in_chat: Dict[int, Dict[int, Dict]] = {}
        self._lock = Lock()
    
    def set_chat_state(self, chat_id: int, data: Optional[Dict] = None) -> None:
        with self._lock:
            self._chat_states[chat_id] = {**self.default_data, **(data or {})}
    
    def get_chat_state(self, chat_id: int) -> Optional[Dict]:
        with self._lock:
            return self._chat_states.get(chat_id)
    
    def set_user_state_in_chat(self, chat_id: int, user_id: int, data: Optional[Dict] = None) -> None:
        with self._lock:
            if chat_id not in self._user_states_in_chat:
                self._user_states_in_chat[chat_id] = {}
            self._user_states_in_chat[chat_id][user_id] = {**self.default_data, **(data or {})}
    
    def get_user_state_in_chat(self, chat_id: int, user_id: int) -> Optional[Dict]:
        with self._lock:
            return self._user_states_in_chat.get(chat_id, {}).get(user_id)
    
    def clear_chat_state(self, chat_id: int) -> bool:
        with self._lock:
            if chat_id in self._chat_states:
                del self._chat_states[chat_id]
                if chat_id in self._user_states_in_chat:
                    del self._user_states_in_chat[chat_id]
                return True
            return False
    
    def clear_user_state_in_chat(self, chat_id: int, user_id: int) -> bool:
        with self._lock:
            if chat_id in self._user_states_in_chat and user_id in self._user_states_in_chat[chat_id]:
                del self._user_states_in_chat[chat_id][user_id]
                return True
            return False


class StateMachine:
    def __init__(self):
        self._states: Dict[str, Union[State, TimedState, ChatState]] = {}
        self._lock = Lock()
    
    def add_state(self, state: Union[State, TimedState, ChatState]) -> None:
        with self._lock:
            self._states[state.name] = state
    
    def get_state(self, name: str) -> Optional[Union[State, TimedState, ChatState]]:
        with self._lock:
            return self._states.get(name)
    
    def remove_state(self, name: str) -> bool:
        with self._lock:
            if name in self._states:
                del self._states[name]
                return True
            return False
    
    def clear_all_states(self) -> None:
        with self._lock:
            self._states.clear()


class AsyncState(State):
    async def set_state(self, user_id: int, data: Optional[Dict] = None) -> None:
        async with asyncio.Lock():
            self._user_states[user_id] = {**self.default_data, **(data or {})}
    
    async def get_state(self, user_id: int) -> Optional[Dict]:
        async with asyncio.Lock():
            return self._user_states.get(user_id)
    
    async def update_state(self, user_id: int, updates: Dict) -> bool:
        async with asyncio.Lock():
            if user_id not in self._user_states:
                return False
            self._user_states[user_id].update(updates)
            return True
    
    async def clear_state(self, user_id: int) -> bool:
        async with asyncio.Lock():
            if user_id in self._user_states:
                del self._user_states[user_id]
                return True
            return False
    
    async def has_state(self, user_id: int) -> bool:
        async with asyncio.Lock():
            return user_id in self._user_states


class AsyncChatState(ChatState):
    async def set_chat_state(self, chat_id: int, data: Optional[Dict] = None) -> None:
        async with asyncio.Lock():
            self._chat_states[chat_id] = {**self.default_data, **(data or {})}
    
    async def get_chat_state(self, chat_id: int) -> Optional[Dict]:
        async with asyncio.Lock():
            return self._chat_states.get(chat_id)
    
    async def set_user_state_in_chat(self, chat_id: int, user_id: int, data: Optional[Dict] = None) -> None:
        async with asyncio.Lock():
            if chat_id not in self._user_states_in_chat:
                self._user_states_in_chat[chat_id] = {}
            self._user_states_in_chat[chat_id][user_id] = {**self.default_data, **(data or {})}
    
    async def get_user_state_in_chat(self, chat_id: int, user_id: int) -> Optional[Dict]:
        async with asyncio.Lock():
            return self._user_states_in_chat.get(chat_id, {}).get(user_id)
    
    async def clear_chat_state(self, chat_id: int) -> bool:
        async with asyncio.Lock():
            if chat_id in self._chat_states:
                del self._chat_states[chat_id]
                if chat_id in self._user_states_in_chat:
                    del self._user_states_in_chat[chat_id]
                return True
            return False
    
    async def clear_user_state_in_chat(self, chat_id: int, user_id: int) -> bool:
        async with asyncio.Lock():
            if chat_id in self._user_states_in_chat and user_id in self._user_states_in_chat[chat_id]:
                del self._user_states_in_chat[chat_id][user_id]
                return True
            return False


class StateManager:
    def __init__(self):
        self._state_machine = StateMachine()
        self._async_state_machine = StateMachine()
    
    def register_state(self, state: Union[State, TimedState, ChatState]) -> None:
        self._state_machine.add_state(state)
    
    def register_async_state(self, state: Union[AsyncState, AsyncChatState]) -> None:
        self._async_state_machine.add_state(state)
    
    def get_state(self, name: str) -> Optional[Union[State, TimedState, ChatState]]:
        return self._state_machine.get_state(name)
    
    async def get_async_state(self, name: str) -> Optional[Union[AsyncState, AsyncChatState]]:
        return self._async_state_machine.get_state(name)
    
    def clear_all_states(self) -> None:
        self._state_machine.clear_all_states()
        self._async_state_machine.clear_all_states()


from typing import List, Optional, Dict, Any

class User:
    def __init__(self, user_data: dict):
        self.id = user_data.get("id")
        self.is_bot = user_data.get("is_bot")
        self.first_name = user_data.get("first_name")
        self.last_name = user_data.get("last_name")
        self.username = user_data.get("username")
        self.language_code = user_data.get("language_code")

    def __str__(self):
        return (
            "User(\n"
            f"    id={self.id},\n"
            f"    is_bot={self.is_bot},\n"
            f"    first_name={self.first_name},\n"
            f"    last_name={self.last_name},\n"
            f"    username={self.username},\n"
            f"    language_code={self.language_code}\n"
            ")"
        )


class Chat:
    def __init__(self, chat_data: dict):
        self.id = chat_data.get("id")
        self.type = chat_data.get("type")
        self.title = chat_data.get("title")
        self.username = chat_data.get("username")
        self.photo = chat_data.get("photo")
        self.description = chat_data.get("description")
        self.invite_link = chat_data.get("invite_link")
        self.permissions = chat_data.get("permissions")

    def __str__(self):
        return (
            "Chat(\n"
            f"    id={self.id},\n"
            f"    type={self.type},\n"
            f"    title={self.title},\n"
            f"    username={self.username},\n"
            f"    photo={self.photo},\n"
            f"    description={self.description},\n"
            f"    invite_link={self.invite_link},\n"
            f"    permissions={self.permissions}\n"
            ")"
        )


class ChatMember:
    def __init__(self, chat_member_data: dict):
        self.user = User(chat_member_data.get("user", {}))
        self.status = chat_member_data.get("status")
        self.custom_title = chat_member_data.get("custom_title")
        self.until_date = chat_member_data.get("until_date")
        self.can_be_edited = chat_member_data.get("can_be_edited")
        self.can_post_messages = chat_member_data.get("can_post_messages")
        self.can_edit_messages = chat_member_data.get("can_edit_messages")
        self.can_delete_messages = chat_member_data.get("can_delete_messages")
        self.can_restrict_members = chat_member_data.get("can_restrict_members")
        self.can_promote_members = chat_member_data.get("can_promote_members")
        self.can_change_info = chat_member_data.get("can_change_info")
        self.can_invite_users = chat_member_data.get("can_invite_users")
        self.can_pin_messages = chat_member_data.get("can_pin_messages")
        self.is_member = chat_member_data.get("is_member")
        self.can_send_messages = chat_member_data.get("can_send_messages")
        self.can_send_media_messages = chat_member_data.get(
            "can_send_media_messages"
        )
        self.can_send_polls = chat_member_data.get("can_send_polls")
        self.can_send_other_messages = chat_member_data.get(
            "can_send_other_messages"
        )
        self.can_add_web_page_previews = chat_member_data.get(
            "can_add_web_page_previews"
        )

    def __str__(self):
        return (
            "ChatMember(\n"
            f"    user={self.user},\n"
            f"    status={self.status},\n"
            f"    custom_title={self.custom_title},\n"
            f"    until_date={self.until_date},\n"
            f"    can_be_edited={self.can_be_edited},\n"
            f"    can_post_messages={self.can_post_messages},\n"
            f"    can_edit_messages={self.can_edit_messages},\n"
            f"    can_delete_messages={self.can_delete_messages},\n"
            f"    can_restrict_members={self.can_restrict_members},\n"
            f"    can_promote_members={self.can_promote_members},\n"
            f"    can_change_info={self.can_change_info},\n"
            f"    can_invite_users={self.can_invite_users},\n"
            f"    can_pin_messages={self.can_pin_messages},\n"
            f"    is_member={self.is_member},\n"
            f"    can_send_messages={self.can_send_messages},\n"
            f"    can_send_media_messages={self.can_send_media_messages},\n"
            f"    can_send_polls={self.can_send_polls},\n"
            f"    can_send_other_messages={self.can_send_other_messages},\n"
            f"    can_add_web_page_previews={self.can_add_web_page_previews}\n"
            ")"
        )


class PhotoSize:
    def __init__(self, photo_data: dict):
        self.file_id = photo_data.get("file_id")
        self.file_unique_id = photo_data.get("file_unique_id")
        self.width = photo_data.get("width")
        self.height = photo_data.get("height")
        self.file_size = photo_data.get("file_size")

    def __str__(self):
        return (
            "PhotoSize(\n"
            f"    file_id={self.file_id},\n"
            f"    file_unique_id={self.file_unique_id},\n"
            f"    width={self.width},\n"
            f"    height={self.height},\n"
            f"    file_size={self.file_size}\n"
            ")"
        )


class Audio:
    def __init__(self, audio_data: dict):
        self.file_id = audio_data.get("file_id")
        self.file_unique_id = audio_data.get("file_unique_id")
        self.duration = audio_data.get("duration")
        self.performer = audio_data.get("performer")
        self.title = audio_data.get("title")
        self.file_name = audio_data.get("file_name")
        self.mime_type = audio_data.get("mime_type")
        self.file_size = audio_data.get("file_size")
        self.thumbnail = (
            PhotoSize(audio_data.get("thumbnail", {}))
            if audio_data.get("thumbnail")
            else None
        )

    def __str__(self):
        return (
            "Audio(\n"
            f"    file_id={self.file_id},\n"
            f"    file_unique_id={self.file_unique_id},\n"
            f"    duration={self.duration},\n"
            f"    performer={self.performer},\n"
            f"    title={self.title},\n"
            f"    file_name={self.file_name},\n"
            f"    mime_type={self.mime_type},\n"
            f"    file_size={self.file_size},\n"
            f"    thumbnail={self.thumbnail}\n"
            ")"
        )


class Document:
    def __init__(self, document_data: dict):
        self.file_id = document_data.get("file_id")
        self.file_unique_id = document_data.get("file_unique_id")
        self.file_name = document_data.get("file_name")
        self.mime_type = document_data.get("mime_type")
        self.file_size = document_data.get("file_size")
        self.thumbnail = (
            PhotoSize(document_data.get("thumbnail", {}))
            if document_data.get("thumbnail")
            else None
        )

    def __str__(self):
        return (
            "Document(\n"
            f"    file_id={self.file_id},\n"
            f"    file_unique_id={self.file_unique_id},\n"
            f"    file_name={self.file_name},\n"
            f"    mime_type={self.mime_type},\n"
            f"    file_size={self.file_size},\n"
            f"    thumbnail={self.thumbnail}\n"
            ")"
        )


class Voice:
    def __init__(self, voice_data: dict):
        self.file_id = voice_data.get("file_id")
        self.file_unique_id = voice_data.get("file_unique_id")
        self.duration = voice_data.get("duration")
        self.mime_type = voice_data.get("mime_type")
        self.file_size = voice_data.get("file_size")

    def __str__(self):
        return (
            "Voice(\n"
            f"    file_id={self.file_id},\n"
            f"    file_unique_id={self.file_unique_id},\n"
            f"    duration={self.duration},\n"
            f"    mime_type={self.mime_type},\n"
            f"    file_size={self.file_size}\n"
            ")"
        )


class Location:
    def __init__(self, location_data: dict):
        self.longitude = location_data.get("longitude")
        self.latitude = location_data.get("latitude")
        self.horizontal_accuracy = location_data.get("horizontal_accuracy")
        self.live_period = location_data.get("live_period")
        self.heading = location_data.get("heading")
        self.proximity_alert_radius = location_data.get("proximity_alert_radius")

    def __str__(self):
        return (
            "Location(\n"
            f"    longitude={self.longitude},\n"
            f"    latitude={self.latitude},\n"
            f"    horizontal_accuracy={self.horizontal_accuracy},\n"
            f"    live_period={self.live_period},\n"
            f"    heading={self.heading},\n"
            f"    proximity_alert_radius={self.proximity_alert_radius}\n"
            ")"
        )


class Video:
    def __init__(self, video_data: dict):
        self.file_id = video_data.get("file_id")
        self.file_unique_id = video_data.get("file_unique_id")
        self.width = video_data.get("width")
        self.height = video_data.get("height")
        self.duration = video_data.get("duration")
        self.file_size = video_data.get("file_size")
        self.thumbnail = (
            PhotoSize(video_data.get("thumbnail", {}))
            if video_data.get("thumbnail")
            else None
        )

    def __str__(self):
        return (
            "Video(\n"
            f"    file_id={self.file_id},\n"
            f"    file_unique_id={self.file_unique_id},\n"
            f"    width={self.width},\n"
            f"    height={self.height},\n"
            f"    duration={self.duration},\n"
            f"    file_size={self.file_size},\n"
            f"    thumbnail={self.thumbnail}\n"
            ")"
        )


class Invoice:
    def __init__(self, invoice_data: dict):
        self.title = invoice_data.get("title")
        self.description = invoice_data.get("description")
        self.start_parameter = invoice_data.get("start_parameter")
        self.currency = invoice_data.get("currency")
        self.total_amount = invoice_data.get("total_amount")

    def __str__(self):
        return (
            "Invoice(\n"
            f"    title={self.title},\n"
            f"    description={self.description},\n"
            f"    start_parameter={self.start_parameter},\n"
            f"    currency={self.currency},\n"
            f"    total_amount={self.total_amount}\n"
            ")"
        )


class Sticker:
    def __init__(self, sticker_data: dict):
        self.file_id = sticker_data.get("file_id")
        self.file_unique_id = sticker_data.get("file_unique_id")
        self.width = sticker_data.get("width")
        self.height = sticker_data.get("height")
        self.is_animated = sticker_data.get("is_animated")
        self.is_video = sticker_data.get("is_video")
        self.emoji = sticker_data.get("emoji")
        self.set_name = sticker_data.get("set_name")
        self.mask_position = sticker_data.get("mask_position")
        self.file_size = sticker_data.get("file_size")
        self.thumbnail = (
            PhotoSize(sticker_data.get("thumbnail", {}))
            if sticker_data.get("thumbnail")
            else None
        )

    def __str__(self):
        return (
            "Sticker(\n"
            f"    file_id={self.file_id},\n"
            f"    file_unique_id={self.file_unique_id},\n"
            f"    width={self.width},\n"
            f"    height={self.height},\n"
            f"    is_animated={self.is_animated},\n"
            f"    is_video={self.is_video},\n"
            f"    emoji={self.emoji},\n"
            f"    set_name={self.set_name},\n"
            f"    mask_position={self.mask_position},\n"
            f"    file_size={self.file_size},\n"
            f"    thumbnail={self.thumbnail}\n"
            ")"
        )


class CallbackQuery:
    def __init__(self, callback_query_data: dict):
        self.id = callback_query_data.get("id")
        self.from_user = User(callback_query_data.get("from", {}))
        message_data = callback_query_data.get("message", {})
        self.message = Message(message_data) if message_data else None
        self.inline_message_id = callback_query_data.get("inline_message_id")
        self.chat_instance = callback_query_data.get("chat_instance")
        self.data = callback_query_data.get("data")
        self.game_short_name = callback_query_data.get("game_short_name")

    def __str__(self):
        return (
            "CallbackQuery(\n"
            f"    id={self.id},\n"
            f"    from_user={self.from_user},\n"
            f"    message={self.message},\n"
            f"    inline_message_id={self.inline_message_id},\n"
            f"    chat_instance={self.chat_instance},\n"
            f"    data={self.data},\n"
            f"    game_short_name={self.game_short_name}\n"
            ")"
        )


class Message:
    def __init__(self, message_data: dict):
        self.message_id = message_data.get("message_id")
        self.from_user = (
            User(message_data.get("from", {}))
            if message_data.get("from")
            else None
        )
        self.date = message_data.get("date")
        self.chat = Chat(message_data.get("chat", {}))
        self.text = message_data.get("text")
        self.photo = (
            [PhotoSize(photo) for photo in message_data.get("photo", [])]
            if message_data.get("photo")
            else None
        )
        self.video = (
            Video(message_data.get("video", {}))
            if message_data.get("video")
            else None
        )
        self.document = (
            Document(message_data.get("document", {}))
            if message_data.get("document")
            else None
        )
        self.audio = (
            Audio(message_data.get("audio", {}))
            if message_data.get("audio")
            else None
        )
        self.voice = (
            Voice(message_data.get("voice", {}))
            if message_data.get("voice")
            else None
        )
        self.caption = message_data.get("caption")
        self.location = (
            Location(message_data.get("location", {}))
            if message_data.get("location")
            else None
        )
        self.reply_to_message = (
            Message(message_data.get("reply_to_message", {}))
            if message_data.get("reply_to_message")
            else None
        )
        self.invoice = (
            Invoice(message_data.get("invoice", {}))
            if message_data.get("invoice")
            else None
        )
        self.sticker = (
            Sticker(message_data.get("sticker", {}))
            if message_data.get("sticker")
            else None
        )
        self.new_chat_members = (
            [User(member) for member in message_data.get("new_chat_members", [])]
            if message_data.get("new_chat_members")
            else None
        )
        self.left_chat_member = (
            User(message_data.get("left_chat_member", {}))
            if message_data.get("left_chat_member")
            else None
        )
        self.forward_from = (
            User(message_data.get("forward_from", {}))
            if message_data.get("forward_from")
            else None
        )
        self.forward_from_chat = (
            Chat(message_data.get("forward_from_chat", {}))
            if message_data.get("forward_from_chat")
            else None
        )
        self.forward_from_message_id = message_data.get("forward_from_message_id")
        self.forward_signature = message_data.get("forward_signature")
        self.forward_sender_name = message_data.get("forward_sender_name")
        self.forward_date = message_data.get("forward_date")

    def __str__(self):
        return (
            "Message(\n"
            f"    message_id={self.message_id},\n"
            f"    from_user={self.from_user},\n"
            f"    date={self.date},\n"
            f"    chat={self.chat},\n"
            f"    text={self.text},\n"
            f"    caption={self.caption},\n"
            f"    audio={self.audio},\n"
            f"    document={self.document},\n"
            f"    photo={self.photo},\n"
            f"    voice={self.voice},\n"
            f"    location={self.location},\n"
            f"    reply_to_message={self.reply_to_message},\n"
            f"    invoice={self.invoice},\n"
            f"    sticker={self.sticker}\n"
            ")"
        )


class UpdateWrapper:
    def __init__(self, update: dict):
        self.update = update
        self.update_id = update.get("update_id")
        self.message = (
            Message(update.get("message", {}))
            if update.get("message")
            else None
        )
        callback_query_data = update.get("callback_query", {})
        self.callback_query = (
            CallbackQuery(callback_query_data)
            if callback_query_data
            else None
        )

    def __str__(self):
        return (
            "UpdateWrapper(\n"
            f"    update_id={self.update_id},\n"
            f"    message={self.message},\n"
            f"    callback_query={self.callback_query}\n"
            ")"
        )


from typing import List, Tuple, Union
import asyncio
import re
import json


class Filter:
    def __init__(self, filter_func):
        self.filter_func = filter_func

    def __call__(self, update):
        return self.filter_func(update)

    def __and__(self, other):
        return Filter(lambda update: self(update) and other(update))

    def __or__(self, other):
        return Filter(lambda update: self(update) or other(update))


class Filters:
    def __init__(self, bot):
        self.bot = bot

    def state(self, state: str):
        return Filter(
            lambda update: (
                ("message" in update 
                 and "from" in update["message"] 
                 and self.bot.get_user_state(update["message"]["from"]["id"]) == state)
                or
                ("callback_query" in update 
                 and "from" in update["callback_query"]
                 and self.bot.get_user_state(update["callback_query"]["from"]["id"]) == state)
            )
        )

    def any_message(self):
        return Filter(lambda update: "message" in update)

    def private(self):
        return Filter(
            lambda update: (
                "message" in update
                and "chat" in update["message"]
                and update["message"]["chat"]["type"] == "private"
            )
        )

    def group(self):
        return Filter(
            lambda update: (
                "message" in update
                and "chat" in update["message"]
                and update["message"]["chat"]["type"] == "group"
            )
        )

    def channel(self):
        return Filter(
            lambda update: (
                "message" in update
                and "chat" in update["message"]
                and update["message"]["chat"]["type"] == "channel"
            )
        )

    def text(self):
        return Filter(
            lambda update: "message" in update and "text" in update["message"]
        )

    def video(self):
        return Filter(
            lambda update: "message" in update and "video" in update["message"]
        )

    def location(self):
        return Filter(
            lambda update: "message" in update and "location" in update["message"]
        )

    def photo(self):
        return Filter(
            lambda update: "message" in update and "photo" in update["message"]
        )

    def reply(self):
        return Filter(
            lambda update: (
                "message" in update
                and "reply_to_message" in update["message"]
            )
        )

    def supergroup_chat_created(self):
        return Filter(
            lambda update: (
                "message" in update
                and "supergroup_chat_created" in update["message"]
            )
        )

    def pinned_message(self):
        return Filter(
            lambda update: (
                "message" in update
                and "pinned_message" in update["message"]
            )
        )

    def new_chat_title(self):
        return Filter(
            lambda update: (
                "message" in update
                and "new_chat_title" in update["message"]
            )
        )

    def new_chat_photo(self):
        return Filter(
            lambda update: (
                "message" in update
                and "new_chat_photo" in update["message"]
            )
        )

    def new_chat_members(self):
        return Filter(
            lambda update: (
                "message" in update
                and "new_chat_members" in update["message"]
            )
        )

    def media(self):
        return Filter(
            lambda update: (
                "message" in update
                and (
                    "photo" in update["message"]
                    or "video" in update["message"]
                    or "document" in update["message"]
                    or "audio" in update["message"]
                    or "voice" in update["message"]
                )
            )
        )

    def left_chat_member(self):
        return Filter(
            lambda update: (
                "message" in update
                and "left_chat_member" in update["message"]
            )
        )

    def group_chat_created(self):
        return Filter(
            lambda update: (
                "message" in update
                and "group_chat_created" in update["message"]
            )
        )

    def forward(self):
        return Filter(
            lambda update: (
                "message" in update
                and "forward_from" in update["message"]
            )
        )

    def document(self):
        return Filter(
            lambda update: (
                "message" in update
                and "document" in update["message"]
            )
        )

    def contact(self):
        return Filter(
            lambda update: (
                "message" in update
                and "contact" in update["message"]
            )
        )

    def channel_chat_created(self):
        return Filter(
            lambda update: (
                "message" in update
                and "channel_chat_created" in update["message"]
            )
        )

    def caption(self):
        return Filter(
            lambda update: (
                "message" in update
                and "caption" in update["message"]
            )
        )

    def all(self):
        return Filter(lambda update: True)

    def audio(self):
        return Filter(
            lambda update: "message" in update and "audio" in update["message"]
        )

    def sticker(self):
        return Filter(
            lambda update: "message" in update and "sticker" in update["message"]
        )

    def voice(self):
        return Filter(
            lambda update: "message" in update and "voice" in update["message"]
        )

    def command(self, q: str):
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and update["message"]["text"].startswith(f"/{q}")
            )
        )

    def pattern(self, q: str):
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and update["message"]["text"].startswith(f"{q}")
            )
        )

    def fix(self, q: str):
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and re.fullmatch(q, update["message"]["text"])
            )
        )

    def multi_command(self, commands: List[str]):
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and any(
                    update["message"]["text"].startswith(f"/{cmd}")
                    or update["message"]["text"].startswith(cmd)
                    for cmd in commands
                )
            )
        )

    def callback_query(self, data: str = None):
        return Filter(
            lambda update: (
                "callback_query" in update
                and (data is None or update["callback_query"]["data"] == data)
            )
        )

    def callback_query_data_startswith(self, prefix: str):
        return Filter(
            lambda update: (
                "callback_query" in update
                and update["callback_query"]["data"].startswith(prefix)
            )
        )

    def callback_query_all(self):
        return Filter(lambda update: "callback_query" in update)

    def pre_checkout_query(self):
        return Filter(lambda update: "pre_checkout_query" in update)

    def successful_payment(self):
        return Filter(
            lambda update: (
                "message" in update
                and "successful_payment" in update["message"]
            )
        )

    def contains_keywords(self, keywords: List[str]):
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and any(keyword in update["message"]["text"] for keyword in keywords)
            )
        )

    def long_message(self, min_length: int):
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and len(update["message"]["text"]) >= min_length
            )
        )

    def contains_link(self):
        url_pattern = re.compile(
            r'http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\\(\\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+'
        )
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and url_pattern.search(update["message"]["text"])
            )
        )

    def contains_mention(self):
        mention_pattern = re.compile(r'@\w+')
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and mention_pattern.search(update["message"]["text"])
            )
        )

    def contains_emoji(self):
        emoji_pattern = re.compile(
            "["
            "\U0001F600-\U0001F64F"
            "\U0001F300-\U0001F5FF"
            "\U0001F680-\U0001F6FF"
            "\U0001F700-\U0001F77F"
            "\U0001F780-\U0001F7FF"
            "\U0001F800-\U0001F8FF"
            "\U0001F900-\U0001F9FF"
            "\U0001FA00-\U0001FA6F"
            "\U0001FA70-\U0001FAFF"
            "\U00002702-\U000027B0"
            "\U000024C2-\U0001F251"
            "]+"
        )
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and emoji_pattern.search(update["message"]["text"])
            )
        )

    def contains_phone_number(self):
        phone_pattern = re.compile(r'(\+98|0)?9\d{9}')
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and phone_pattern.search(update["message"]["text"])
            )
        )

    def contains_location(self):
        return Filter(
            lambda update: (
                "message" in update
                and "location" in update["message"]
            )
        )

    def contains_file_type(self, file_types: List[str]):
        return Filter(
            lambda update: (
                "message" in update
                and "document" in update["message"]
                and any(
                    update["message"]["document"]["mime_type"].endswith(file_type)
                    for file_type in file_types
                )
            )
        )

    def contains_hashtag(self):
        hashtag_pattern = re.compile(r'#\w+')
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and hashtag_pattern.search(update["message"]["text"])
            )
        )

    def contains_number(self):
        number_pattern = re.compile(r'\d+')
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and number_pattern.search(update["message"]["text"])
            )
        )

    def contains_date(self):
        date_pattern = re.compile(r'\d{4}[-/]\d{2}[-/]\d{2}|\d{2}[-/]\d{2}[-/]\d{4}')
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and date_pattern.search(update["message"]["text"])
            )
        )

    def contains_time(self):
        time_pattern = re.compile(r'\d{1,2}:\d{2}(?::\d{2})?(?:\s?[AP]M)?', re.IGNORECASE)
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and time_pattern.search(update["message"]["text"])
            )
        )

    def contains_postal_code(self):
        postal_code_pattern = re.compile(r'\b\d{10}\b')
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and postal_code_pattern.search(update["message"]["text"])
            )
        )

    def contains_national_code(self):
        national_code_pattern = re.compile(r'\b\d{10}\b')
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and national_code_pattern.search(update["message"]["text"])
            )
        )

    def contains_bank_card(self):
        bank_card_pattern = re.compile(r'\b\d{16}\b')
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and bank_card_pattern.search(update["message"]["text"])
            )
        )

    def contains_hex_color(self):
        hex_color_pattern = re.compile(r'#([a-fA-F0-9]{6}|[a-fA-F0-9]{3})')
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and hex_color_pattern.search(update["message"]["text"])
            )
        )

    def contains_html(self):
        html_pattern = re.compile(r'<[^>]+>')
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and html_pattern.search(update["message"]["text"])
            )
        )

    def contains_json(self):
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and self._is_json(update["message"]["text"])
            )
        )

    def _is_json(self, text: str):
        try:
            json.loads(text)
            return True
        except ValueError:
            return False

    def contains_xml(self):
        xml_pattern = re.compile(r'<[^>]+>[^<]+</[^>]+>')
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and xml_pattern.search(update["message"]["text"])
            )
        )

    def contains_sql(self):
        sql_pattern = re.compile(r'\b(SELECT|INSERT|UPDATE|DELETE|CREATE|DROP|ALTER)\b', re.IGNORECASE)
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and sql_pattern.search(update["message"]["text"])
            )
        )

    def contains_code(self):
        code_pattern = re.compile(r'\b(def|function|class|import|console\.log|print)\b', re.IGNORECASE)
        return Filter(
            lambda update: (
                "message" in update
                and "text" in update["message"]
                and code_pattern.search(update["message"]["text"])
            )
        )

    @property
    def pre_checkout_query_id(self):
        return self.pre_checkout_query.get("id")

    @property
    def pre_checkout_query_from(self):
        return self.pre_checkout_query.get("from", {})

    @property
    def pre_checkout_query_currency(self):
        return self.pre_checkout_query.get("currency")

    @property
    def pre_checkout_query_total_amount(self):
        return self.pre_checkout_query.get("total_amount")

    @property
    def pre_checkout_query_invoice_payload(self):
        return self.pre_checkout_query.get("invoice_payload")

    @property
    def successful_payment_currency(self):
        return self.successful_payment.get("currency")

    @property
    def successful_payment_total_amount(self):
        return self.successful_payment.get("total_amount")

    @property
    def successful_payment_invoice_payload(self):
        return self.successful_payment.get("invoice_payload")

    @property
    def successful_payment_telegram_payment_charge_id(self):
        return self.successful_payment.get("telegram_payment_charge_id")


class ReplyKeyboardMarkup:
    def __init__(self, keyboard=None, selective=None):
        self.keyboard = keyboard if keyboard else []
        self.selective = selective

    def add_row(self, *buttons):
        self.keyboard.append([button for button in buttons])
        return self

    def to_dict(self):
        result = {
            "keyboard": [
                [button.to_dict() for button in row] 
                for row in self.keyboard
            ]
        }
        if self.selective is not None:
            result["selective"] = self.selective
        return result

class KeyboardButton:
    def __init__(self, text, request_contact=False, request_location=False, web_app=None):
        self.text = text
        self.request_contact = request_contact
        self.request_location = request_location
        self.web_app = web_app

    def to_dict(self):
        button_dict = {"text": self.text}
        if self.request_contact:
            button_dict["request_contact"] = True
        if self.request_location:
            button_dict["request_location"] = True
        if self.web_app:
            button_dict["web_app"] = self.web_app.to_dict()
        return button_dict

class KeyboardButtonPollType:
    def __init__(self, type=None):
        self.type = type

    def to_dict(self):
        return {"type": self.type} if self.type else {}

class WebAppInfo:
    def __init__(self, url):
        self.url = url

    def to_dict(self):
        return {"url": self.url}


class InlineKeyboardMarkup:
    def __init__(self, keyboard=None):
        self.keyboard = keyboard if keyboard else []

    def add_row(self, *buttons):
        self.keyboard.append([button for button in buttons])
        return self

    def to_dict(self):
        return {
            "inline_keyboard": [
                [button.to_dict() for button in row] 
                for row in self.keyboard
            ]
        }

class InlineKeyboardButton:
    def __init__(self, text, callback_data=None, url=None, web_app=None):
        self.text = text
        self.callback_data = callback_data
        self.url = url
        self.web_app = web_app

    def to_dict(self):
        button_dict = {"text": self.text}
        
        if self.callback_data:
            button_dict["callback_data"] = self.callback_data
        if self.url:
            button_dict["url"] = self.url
        if self.web_app:
            button_dict["web_app"] = self.web_app.to_dict()
            
        return button_dict

class WebAppInfo:
    def __init__(self, url):
        self.url = url

    def to_dict(self):
        return {"url": self.url}